/*
Sector de arranque en GNU Assembler
Author: Erwin Meza Vega

Description:

Este sector de arranque de 512 bytes es cargado por la BIOS en la posicion de
memoria 0x7C00. Luego, la BIOS realiza un jmp (jump)  al codigo leido en esta
direccion de memoria. De forma similar a un 'goto' en algunos lenguajes de
programacion de alto nivel, el jmp permite 'continuar la ejecucion'  o
'pasar el control' al codigo ubicado en otra posicion de memoria.

Generalmente el jmp que realiza la BIOS tiene el siguiente formato:
    ljmp 0x0000 : 0x7C00
    jmp far 0x0000 : 0x7c00

Este formato de jmp (long jmp) modifica los registros cs e ip, de la siguiente
forma:
cs = 0x0000 => Registro de segmento de codigo ( Code Segment)
ip = 0x7C00 => Registro Instruction Pointer

De esta forma, cs:ip apunta a la direccion de memoria 0x7C00, en la cual se
encuentra el codigo del bootsector.

La diferencia entre 'jmp' y 'ljmp' consiste en que jmp solo modifica el
registro ip, mientras que ljmp modifica al mismo tiempo  los registros
cs e ip.

*/

.intel_syntax noprefix /* Sintaxis Intel, evitar los prefijos en los registros*/
.code16 # 16 bits- Modo Real

.section .text  #Seccion de texto

.globl start

/* Simbolo de entrada del bootsector. En este punto comienza su ejecucion. */
start:

    /* Asegurarse que cs tome el valor de 0x7C0 ip toma el valor del
    desplazamiento de la etiqueta 'entry_point' */
    ljmp 0x7C0 : OFFSET entry_point

/* La ejecucion continua en este punto, pero se garantiza que el valor de cs
es 0x7C0*/
entry_point:

    /* Apuntar el registro de segmento DS al mismo segmento que CS*/
    mov ax, cs
    mov ds, ax

    xor ax, ax
    int 0x16

    mov ah, 0x0E
    int 0x10

    jmp finished /* Saltar a el simbolo (la etiqueta) 'finished'


/* Simbolo (etiqueta) finished. */
finished:
   jmp finished   /* Salta a la etiqueta 'finished' (ciclo infinito) */

/*
Fin del bootsector
*/

/*
Una vez compilado, el codigo del bootsector no puede ocupar mas de 510 bytes.
Al generar el codigo ejecutable del bootsector, el linker se encarga
de rellenar con ceros el espacio a partir de la ultima instruccion de codigo
compilado hasta completar 510 bytes, y luego incluye los bytes 0x55 y 0xAA
para generar un bootsector valido de 512 bytes.

Ver el archivo link_bootsector.ld para mas detalles.

*/
